/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import java.util.List;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ArrayContentProvider;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.ComboBoxViewerCellEditor;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * This class constructs and manages an editing element for the annotation view. It provides a combo
 * box that displays predefined, fixed values from an annotation value provider.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class AnnotationEditingSupportCombo extends EditingSupport {
	/** Combo box cell editor */
	private ComboBoxViewerCellEditor cellEditor = null;

	/** Specification class of this column */
	private Class<? extends IAnnotatedSpecification> specClass;

	/**
	 * Constructor.
	 * 
	 * @param viewer
	 *            the column viewer
	 * @param values
	 *            the values for the combo box
	 * @param class1
	 */
	public AnnotationEditingSupportCombo(ColumnViewer viewer, List<String> values,
			Class<? extends IAnnotatedSpecification> class1) {
		super(viewer);
		cellEditor = new ComboBoxViewerCellEditor((Composite)getViewer().getControl());
		cellEditor.setLabelProvider(new LabelProvider());
		cellEditor.setContentProvider(new ArrayContentProvider());
		cellEditor.setInput(values);
		specClass = class1;
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {
		return cellEditor;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry data = (AnnotationEntry)element;
			return data.getSpecificationValue(specClass) != AnnotationEntry.NOVAL;
		}

		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry data = (AnnotationEntry)element;
			return data.getSpecificationValue(specClass);
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, final Object value) {
		if(element instanceof AnnotationEntry && value instanceof String) {
			final AnnotationEntry data = (AnnotationEntry)element;

			if(!value.equals(data.getSpecificationValue(specClass))) {
				ITopLevelElement modelContext =
						IPersistencyService.INSTANCE.getTopLevelElementFor(data.getModelElement());
				modelContext.runAsCommand(new Runnable() {

					@Override
					public void run() {

						try {
							data.setSpecificationValue((String)value, specClass);
						} catch(IllegalArgumentException e) {
							// should not happen with combo boxes and given values
							MessageDialog
									.openError(
											new Shell(),
											"ERROR",
											"The value you entered does not have the desired type.\nDetailed message:\n" +
													e.getMessage());
						} catch(Exception e) {
							// e.printStackTrace();
							MessageDialog.openError(new Shell(), "ERROR", e.getMessage());
						}
					}
				});
				this.getViewer().refresh(true);
			}

		}
	}
}
