/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2014 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.base.ui.annotation;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.CellEditor;
import org.eclipse.jface.viewers.ColumnViewer;
import org.eclipse.jface.viewers.EditingSupport;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Shell;
import org.fortiss.tooling.base.model.element.IAnnotatedSpecification;
import org.fortiss.tooling.kernel.extension.data.ITopLevelElement;
import org.fortiss.tooling.kernel.service.IPersistencyService;

/**
 * This class constructs and manages an editing element for the annotation view. It provides a
 * simple text input field, for the case that not predefined values are given and no custom editing
 * fields are defined.
 * 
 * @author diewald
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating RED Hash:
 */
public class AnnotationEditingSupportStandard extends EditingSupport {

	/** Text cell editor */
	protected TextCellEditor cellEditor = null;
	/** Specification class of this column */
	protected Class<? extends IAnnotatedSpecification> specClass;

	/** Constructor. */
	public AnnotationEditingSupportStandard(ColumnViewer viewer,
			Class<? extends IAnnotatedSpecification> class1) {
		super(viewer);

		cellEditor = new TextCellEditor((Composite)getViewer().getControl(), SWT.NONE);
		specClass = class1;
	}

	/** {@inheritDoc} */
	@Override
	protected CellEditor getCellEditor(Object element) {
		return cellEditor;
	}

	/** {@inheritDoc} */
	@Override
	protected boolean canEdit(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry data = (AnnotationEntry)element;
			return data.getSpecificationValue(specClass) != AnnotationEntry.NOVAL;
		}

		return true;
	}

	/** {@inheritDoc} */
	@Override
	protected Object getValue(Object element) {
		if(element instanceof AnnotationEntry) {
			AnnotationEntry data = (AnnotationEntry)element;
			return data.getSpecificationValue(specClass);
		}
		return null;
	}

	/** {@inheritDoc} */
	@Override
	protected void setValue(Object element, final Object value) {
		if(element instanceof AnnotationEntry && value instanceof String) {
			final AnnotationEntry data = (AnnotationEntry)element;

			if(!value.equals(data.getSpecificationValue(specClass))) {
				ITopLevelElement modelContext =
						IPersistencyService.INSTANCE.getTopLevelElementFor(data.getModelElement());
				modelContext.runAsCommand(new Runnable() {

					@Override
					public void run() {
						try {
							data.setSpecificationValue((String)value, specClass);
						} catch(IllegalArgumentException e) {
							// e.printStackTrace();
							MessageDialog
									.openError(
											new Shell(),
											"ERROR",
											"The value you entered does not have the desired type.\nDetailed message:\n" +
													e.getMessage());
						} catch(Exception e) {
							MessageDialog.openError(new Shell(), "ERROR", e.getMessage());
						}
					}
				});
				this.getViewer().refresh(true);
			}

		}
	}

}
